@@ -10,18 +10,40 @@ import numpy as np
 import pandas as pd
 from tqdm import tqdm
 import json
-import cv2
-from keras import backend as K
-import keras
-import tensorflow as tf
 import pickle
 
-from deepface import DeepFace
 from deepface.basemodels import VGGFace, OpenFace, Facenet, FbDeepFace, DeepID
 from deepface.extendedmodels import Age, Gender, Race, Emotion
 from deepface.commons import functions, realtime, distance as dst
 
-def verify(img1_path, img2_path = '', model_name ='VGG-Face', distance_metric = 'cosine', model = None, enforce_detection = True, detector_backend = 'opencv'):
+def build_model(model_name):
+	
+	models = {
+		'VGG-Face': VGGFace.loadModel, 
+		'OpenFace': OpenFace.loadModel,
+		'Facenet': Facenet.loadModel,
+		'DeepFace': FbDeepFace.loadModel,
+		'DeepID': DeepID.loadModel,
+		'Dlib': DlibResNet_,
+		'Emotion': Emotion.loadModel,
+		'Age': Age.loadModel,
+		'Gender': Gender.loadModel,
+		'Race': Race.loadModel
+	}
+
+	model = models.get(model_name)
+	
+	if model:
+		model = model()
+		print('Using {} model backend'.format(model_name))
+		return model
+	else:
+		raise ValueError('Invalid model_name passed - {}'.format(model_name))
+
+def verify(img1_path, img2_path = '', model_name='VGG-Face', distance_metric='cosine', 
+		   model=None, enforce_detection=True, detector_backend = 'opencv'):
+
+	# this is not a must because it is very huge.		
 
 	tic = time.time()
 
@@ -34,6 +56,8 @@ def verify(img1_path, img2_path = '', model_name ='VGG-Face', distance_metric =
 
 	#------------------------------
 	
+	functions.initialize_detector(detector_backend = detector_backend)
+
 	resp_objects = []
 	
 	if model_name == 'Ensemble':
@@ -50,16 +74,16 @@ def verify(img1_path, img2_path = '', model_name ='VGG-Face', distance_metric =
 				
 				if index == 0:
 					model_pbar.set_description("Loading VGG-Face")
-					model["VGG-Face"] = VGGFace.loadModel()
+					model["VGG-Face"] = build_model('VGG-Face')
 				elif index == 1:
 					model_pbar.set_description("Loading Google FaceNet")
-					model["Facenet"] = Facenet.loadModel()
+					model["Facenet"] = build_model('Facenet')
 				elif index == 2:
 					model_pbar.set_description("Loading OpenFace")
-					model["OpenFace"] = OpenFace.loadModel()
+					model["OpenFace"] = build_model('OpenFace')
 				elif index == 3:
 					model_pbar.set_description("Loading Facebook DeepFace")
-					model["DeepFace"] = FbDeepFace.loadModel()
+					model["DeepFace"] = build_model('DeepFace')
 					
 		#--------------------------
 		#validate model dictionary because it might be passed from input as pre-trained
@@ -198,34 +222,9 @@ def verify(img1_path, img2_path = '', model_name ='VGG-Face', distance_metric =
 	#--------------------------------
 	#ensemble learning disabled
 	
-	if model == None:
-		if model_name == 'VGG-Face':
-			print("Using VGG-Face model backend and", distance_metric,"distance.")
-			model = VGGFace.loadModel()
-
-		elif model_name == 'OpenFace':
-			print("Using OpenFace model backend", distance_metric,"distance.")
-			model = OpenFace.loadModel()
-
-		elif model_name == 'Facenet':
-			print("Using Facenet model backend", distance_metric,"distance.")
-			model = Facenet.loadModel()
-
-		elif model_name == 'DeepFace':
-			print("Using FB DeepFace model backend", distance_metric,"distance.")
-			model = FbDeepFace.loadModel()
-		
-		elif model_name == 'DeepID':
-			print("Using DeepID2 model backend", distance_metric,"distance.")
-			model = DeepID.loadModel()
-		
-		elif model_name == 'Dlib':
-			print("Using Dlib ResNet model backend", distance_metric,"distance.")
-			from deepface.basemodels.DlibResNet import DlibResNet #this is not a must because it is very huge.
-			model = DlibResNet()
-
-		else:
-			raise ValueError("Invalid model_name passed - ", model_name)
+	if model == None:		
+		model = build_model(model_name)
+	
 	else: #model != None
 		print("Already built model is passed")
 
@@ -344,7 +343,6 @@ def verify(img1_path, img2_path = '', model_name ='VGG-Face', distance_metric =
 		return resp_obj
 		#return resp_objects
 
-
 def analyze(img_path, actions = [], models = {}, enforce_detection = True, detector_backend = 'opencv'):
 
 	if type(img_path) == list:
@@ -355,6 +353,10 @@ def analyze(img_path, actions = [], models = {}, enforce_detection = True, detec
 		bulkProcess = False
 
 	#---------------------------------
+	
+	functions.initialize_detector(detector_backend = detector_backend)
+	
+	#---------------------------------
 
 	#if a specific target is not passed, then find them all
 	if len(actions) == 0:
@@ -369,28 +371,28 @@ def analyze(img_path, actions = [], models = {}, enforce_detection = True, detec
 			print("already built emotion model is passed")
 			emotion_model = models['emotion']
 		else:
-			emotion_model = Emotion.loadModel()
+			emotion_model = build_model('Emotion')
 
 	if 'age' in actions:
 		if 'age' in models:
 			#print("already built age model is passed")
 			age_model = models['age']
 		else:
-			age_model = Age.loadModel()
+			age_model = build_model('Age')
 
 	if 'gender' in actions:
 		if 'gender' in models:
 			print("already built gender model is passed")
 			gender_model = models['gender']
 		else:
-			gender_model = Gender.loadModel()
+			gender_model = build_model('Gender')
 
 	if 'race' in actions:
 		if 'race' in models:
 			print("already built race model is passed")
 			race_model = models['race']
 		else:
-			race_model = Race.loadModel()
+			race_model = build_model('Race')
 	#---------------------------------
 
 	resp_objects = []
@@ -444,7 +446,7 @@ def analyze(img_path, actions = [], models = {}, enforce_detection = True, detec
 
 			elif action == 'age':
 				if img_224 is None:
-					img_224 = functions.preprocess_face(img_path, target_size = (224, 224), grayscale = False, enforce_detection = enforce_detection) #just emotion model expects grayscale images
+					img_224 = functions.preprocess_face(img = img_path, target_size = (224, 224), grayscale = False, enforce_detection = enforce_detection, detector_backend = detector_backend) #just emotion model expects grayscale images
 				#print("age prediction")
 				age_predictions = age_model.predict(img_224)[0,:]
 				apparent_age = Age.findApparentAge(age_predictions)
@@ -513,8 +515,10 @@ def analyze(img_path, actions = [], models = {}, enforce_detection = True, detec
 		return resp_obj
 		#return resp_objects
 
-
 def detectFace(img_path, detector_backend = 'opencv'):
+	
+	functions.initialize_detector(detector_backend = detector_backend)
+	
 	img = functions.preprocess_face(img = img_path, detector_backend = detector_backend)[0] #preprocess_face returns (1, 224, 224, 3)
 	return img[:, :, ::-1] #bgr to rgb
 
@@ -532,31 +536,19 @@ def find(img_path, db_path, model_name ='VGG-Face', distance_metric = 'cosine',
 		bulkProcess = False
 		img_paths = [img_path]
 	
+	#-------------------------------
+	
+	functions.initialize_detector(detector_backend = detector_backend)
+	
+	#-------------------------------
+	
 	if os.path.isdir(db_path) == True:
 		
 		#---------------------------------------
 		
 		if model == None:
-			if model_name == 'VGG-Face':
-				print("Using VGG-Face model backend and", distance_metric,"distance.")
-				model = VGGFace.loadModel()
-			elif model_name == 'OpenFace':
-				print("Using OpenFace model backend", distance_metric,"distance.")
-				model = OpenFace.loadModel()
-			elif model_name == 'Facenet':
-				print("Using Facenet model backend", distance_metric,"distance.")
-				model = Facenet.loadModel()
-			elif model_name == 'DeepFace':
-				print("Using FB DeepFace model backend", distance_metric,"distance.")
-				model = FbDeepFace.loadModel()
-			elif model_name == 'DeepID':
-				print("Using DeepID model backend", distance_metric,"distance.")
-				model = DeepID.loadModel()
-			elif model_name == 'Dlib':
-				print("Using Dlib ResNet model backend", distance_metric,"distance.")
-				from deepface.basemodels.DlibResNet import DlibResNet #this is not a must because it is very huge
-				model = DlibResNet()
-			elif model_name == 'Ensemble':
+			
+			if model_name == 'Ensemble':
 				print("Ensemble learning enabled")
 				#TODO: include DeepID in ensemble method
 				
@@ -569,19 +561,20 @@ def find(img_path, db_path, model_name ='VGG-Face', distance_metric = 'cosine',
 				for index in pbar:
 					if index == 0:
 						pbar.set_description("Loading VGG-Face")
-						models['VGG-Face'] = VGGFace.loadModel()
+						models['VGG-Face'] = build_model('VGG-Face')
 					elif index == 1:
 						pbar.set_description("Loading FaceNet")
-						models['Facenet'] = Facenet.loadModel()
+						models['Facenet'] = build_model('Facenet')
 					elif index == 2:
 						pbar.set_description("Loading OpenFace")
-						models['OpenFace'] = OpenFace.loadModel()
+						models['OpenFace'] = build_model('OpenFace')
 					elif index == 3:
 						pbar.set_description("Loading DeepFace")
-						models['DeepFace'] = FbDeepFace.loadModel()
-						
-			else:
-				raise ValueError("Invalid model_name passed - ", model_name)	
+						models['DeepFace'] = build_model('DeepFace')
+			
+			else: #model is not ensemble
+				model = build_model(model_name)
+				
 		else: #model != None
 			print("Already built model is passed")
 			
@@ -865,12 +858,15 @@ def find(img_path, db_path, model_name ='VGG-Face', distance_metric = 'cosine',
 	return None
 	
 def stream(db_path = '', model_name ='VGG-Face', distance_metric = 'cosine', enable_face_analysis = True):
+	
+	functions.initialize_detector(detector_backend = 'opencv')
+	
 	realtime.analysis(db_path, model_name, distance_metric, enable_face_analysis)
 
-def allocateMemory():
-	print("Analyzing your system...")
-	functions.allocateMemory()
-
+def DlibResNet_():
+	#this is not a regular Keras model.
+	from deepface.basemodels.DlibResNet import DlibResNet
+	return DlibResNet()
 #---------------------------
 #main
 
