@@ -18,7 +18,86 @@ import tensorflow as tf
 import keras
 import bz2
 from deepface.commons import distance
+from mtcnn import MTCNN #0.1.0
 
+def initialize_detector(detector_backend):
+	
+	global face_detector
+	
+	home = str(Path.home())
+	
+	if detector_backend == 'opencv':
+		opencv_path = get_opencv_path()
+		
+		face_detector_path = opencv_path+"haarcascade_frontalface_default.xml"
+		eye_detector_path = opencv_path+"haarcascade_eye.xml"
+	
+		if os.path.isfile(face_detector_path) != True:
+			raise ValueError("Confirm that opencv is installed on your environment! Expected path ",face_detector_path," violated.")
+		
+		face_detector = cv2.CascadeClassifier(face_detector_path)
+		
+		global eye_detector
+		eye_detector = cv2.CascadeClassifier(eye_detector_path)
+	
+	elif detector_backend == 'ssd':
+	
+		#check required ssd model exists in the home/.deepface/weights folder
+		
+		#model structure
+		if os.path.isfile(home+'/.deepface/weights/deploy.prototxt') != True:
+			
+			print("deploy.prototxt will be downloaded...")
+			
+			url = "https://github.com/opencv/opencv/raw/3.4.0/samples/dnn/face_detector/deploy.prototxt"
+			
+			output = home+'/.deepface/weights/deploy.prototxt'
+			
+			gdown.download(url, output, quiet=False)
+		
+		#pre-trained weights
+		if os.path.isfile(home+'/.deepface/weights/res10_300x300_ssd_iter_140000.caffemodel') != True:
+			
+			print("res10_300x300_ssd_iter_140000.caffemodel will be downloaded...")
+			
+			url = "https://github.com/opencv/opencv_3rdparty/raw/dnn_samples_face_detector_20170830/res10_300x300_ssd_iter_140000.caffemodel"
+			
+			output = home+'/.deepface/weights/res10_300x300_ssd_iter_140000.caffemodel'
+			
+			gdown.download(url, output, quiet=False)
+		
+		face_detector = cv2.dnn.readNetFromCaffe(
+			home+"/.deepface/weights/deploy.prototxt", 
+			home+"/.deepface/weights/res10_300x300_ssd_iter_140000.caffemodel"
+		)
+		
+	elif detector_backend == 'dlib':
+		import dlib #this is not a must library within deepface. that's why, I didn't put this import to a global level. version: 19.20.0
+		
+		global sp
+		
+		face_detector = dlib.get_frontal_face_detector()
+		
+		#check required file exists in the home/.deepface/weights folder
+		if os.path.isfile(home+'/.deepface/weights/shape_predictor_5_face_landmarks.dat') != True:
+			
+			print("shape_predictor_5_face_landmarks.dat.bz2 is going to be downloaded") 
+			
+			url = "http://dlib.net/files/shape_predictor_5_face_landmarks.dat.bz2"
+			output = home+'/.deepface/weights/'+url.split("/")[-1]
+			
+			gdown.download(url, output, quiet=False)
+			
+			zipfile = bz2.BZ2File(output)
+			data = zipfile.read()
+			newfilepath = output[:-4] #discard .bz2 extension
+			open(newfilepath, 'wb').write(data)
+		
+		sp = dlib.shape_predictor(home+"/.deepface/weights/shape_predictor_5_face_landmarks.dat")
+		
+	elif detector_backend == 'mtcnn':
+		face_detector = MTCNN()
+	
 def loadBase64Img(uri):
    encoded_data = uri.split(',')[1]
    nparr = np.fromstring(base64.b64decode(encoded_data), np.uint8)
@@ -39,55 +118,18 @@ def initializeFolder():
 	
 def findThreshold(model_name, distance_metric):
 	
-	threshold = 0.40
+	base_threshold = {'cosine': 0.40, 'euclidean': 0.55, 'euclidean_l2': 0.75}
 	
-	if model_name == 'VGG-Face':
-		if distance_metric == 'cosine':
-			threshold = 0.40
-		elif distance_metric == 'euclidean':
-			threshold = 0.55
-		elif distance_metric == 'euclidean_l2':
-			threshold = 0.75	
-	
-	elif model_name == 'OpenFace':
-		if distance_metric == 'cosine':
-			threshold = 0.10
-		elif distance_metric == 'euclidean':
-			threshold = 0.55
-		elif distance_metric == 'euclidean_l2':
-			threshold = 0.55
-	
-	elif model_name == 'Facenet':
-		if distance_metric == 'cosine':
-			threshold = 0.40
-		elif distance_metric == 'euclidean':
-			threshold = 10
-		elif distance_metric == 'euclidean_l2':
-			threshold = 0.80
-	
-	elif model_name == 'DeepFace':
-		if distance_metric == 'cosine':
-			threshold = 0.23
-		elif distance_metric == 'euclidean':
-			threshold = 64
-		elif distance_metric == 'euclidean_l2':
-			threshold = 0.64
-	
-	elif model_name == 'DeepID':
-		if distance_metric == 'cosine':
-			threshold = 0.015
-		elif distance_metric == 'euclidean':
-			threshold = 45
-		elif distance_metric == 'euclidean_l2':
-			threshold = 0.17
-	
-	elif model_name == 'Dlib':
-		if distance_metric == 'cosine':
-			threshold = 0.07
-		elif distance_metric == 'euclidean':
-			threshold = 0.60
-		elif distance_metric == 'euclidean_l2':
-			threshold = 0.60
+	thresholds = {
+		'VGG-Face': {'cosine': 0.40, 'euclidean': 0.55, 'euclidean_l2': 0.75},
+		'OpenFace': {'cosine': 0.10, 'euclidean': 0.55, 'euclidean_l2': 0.55},
+		'Facenet':  {'cosine': 0.40, 'euclidean': 10, 'euclidean_l2': 0.80},
+		'DeepFace': {'cosine': 0.23, 'euclidean': 64, 'euclidean_l2': 0.64},
+		'DeepID': 	{'cosine': 0.015, 'euclidean': 45, 'euclidean_l2': 0.17},
+		'Dlib': 	{'cosine': 0.07, 'euclidean': 0.6, 'euclidean_l2': 0.6}
+		}
+
+	threshold = thresholds.get(model_name, base_threshold).get(distance_metric, 0.4)
 	
 	return threshold
 
@@ -128,18 +170,11 @@ def detect_face(img, detector_backend = 'opencv', grayscale = False, enforce_det
 	
 	home = str(Path.home())
 	
-	if detector_backend == 'opencv':
-	
-		#get opencv configuration up first
-		opencv_path = get_opencv_path()
-		face_detector_path = opencv_path+"haarcascade_frontalface_default.xml"
-	
-		if os.path.isfile(face_detector_path) != True:
-			raise ValueError("Confirm that opencv is installed on your environment! Expected path ",face_detector_path," violated.")
+	#if functions.preproces_face is called directly, then face_detector global variable might not been initialized.
+	if not "face_detector" in globals():
+		initialize_detector(detector_backend = detector_backend)
 		
-		face_detector = cv2.CascadeClassifier(face_detector_path)
-	
-		#--------------------------
+	if detector_backend == 'opencv':
 		
 		faces = []
 		
@@ -163,39 +198,6 @@ def detect_face(img, detector_backend = 'opencv', grayscale = False, enforce_det
 
 	elif detector_backend == 'ssd':
 		
-		#---------------------------
-		#check required ssd model exists in the home/.deepface/weights folder
-		
-		#model structure
-		if os.path.isfile(home+'/.deepface/weights/deploy.prototxt') != True:
-			
-			print("deploy.prototxt will be downloaded...")
-			
-			url = "https://github.com/opencv/opencv/raw/3.4.0/samples/dnn/face_detector/deploy.prototxt"
-			
-			output = home+'/.deepface/weights/deploy.prototxt'
-			
-			gdown.download(url, output, quiet=False)
-			
-		
-		#pre-trained weights
-		if os.path.isfile(home+'/.deepface/weights/res10_300x300_ssd_iter_140000.caffemodel') != True:
-			
-			print("res10_300x300_ssd_iter_140000.caffemodel will be downloaded...")
-			
-			url = "https://github.com/opencv/opencv_3rdparty/raw/dnn_samples_face_detector_20170830/res10_300x300_ssd_iter_140000.caffemodel"
-			
-			output = home+'/.deepface/weights/res10_300x300_ssd_iter_140000.caffemodel'
-			
-			gdown.download(url, output, quiet=False)
-			
-		#---------------------------
-		
-		ssd_detector = cv2.dnn.readNetFromCaffe(
-			home+"/.deepface/weights/deploy.prototxt", 
-			home+"/.deepface/weights/res10_300x300_ssd_iter_140000.caffemodel"
-		)
-		
 		ssd_labels = ["img_id", "is_face", "confidence", "left", "top", "right", "bottom"]
 		
 		target_size = (300, 300)
@@ -211,8 +213,8 @@ def detect_face(img, detector_backend = 'opencv', grayscale = False, enforce_det
 		
 		imageBlob = cv2.dnn.blobFromImage(image = img)
 		
-		ssd_detector.setInput(imageBlob)
-		detections = ssd_detector.forward()
+		face_detector.setInput(imageBlob)
+		detections = face_detector.forward()
 		
 		detections_df = pd.DataFrame(detections[0][0], columns = ssd_labels)
 		
@@ -250,11 +252,8 @@ def detect_face(img, detector_backend = 'opencv', grayscale = False, enforce_det
 				raise ValueError("Face could not be detected. Please confirm that the picture is a face photo or consider to set enforce_detection param to False.")
 	
 	elif detector_backend == 'dlib':
-		import dlib #this is not a must library within deepface. that's why, I didn't put this import to a global level. version: 19.20.0
-		
-		detector = dlib.get_frontal_face_detector()
-		
-		detections = detector(img, 1)
+
+		detections = face_detector(img, 1)
 		
 		if len(detections) > 0:
 			
@@ -275,10 +274,9 @@ def detect_face(img, detector_backend = 'opencv', grayscale = False, enforce_det
 				raise ValueError("Face could not be detected. Please confirm that the picture is a face photo or consider to set enforce_detection param to False.") 
 		
 	elif detector_backend == 'mtcnn':
-		from mtcnn import MTCNN #0.1.0
-		mtcnn_detector = MTCNN()
 		
-		detections = mtcnn_detector.detect_faces(img)
+		img_rgb = cv2.cvtColor(img, cv2.COLOR_BGR2RGB) #mtcnn expects RGB but OpenCV read BGR
+		detections = face_detector.detect_faces(img_rgb)
 		
 		if len(detections) > 0:
 			detection = detections[0]
@@ -287,7 +285,7 @@ def detect_face(img, detector_backend = 'opencv', grayscale = False, enforce_det
 			return detected_face
 		
 		else: #if no face detected
-			if enforce_detection != True:			
+			if not enforce_detection:			
 				return img
 	
 			else:
@@ -296,8 +294,6 @@ def detect_face(img, detector_backend = 'opencv', grayscale = False, enforce_det
 	else:
 		detectors = ['opencv', 'ssd', 'dlib', 'mtcnn']
 		raise ValueError("Valid backends are ", detectors," but you passed ", detector_backend)
-	
-	return 0
 
 def alignment_procedure(img, left_eye, right_eye):
 		
@@ -351,10 +347,6 @@ def align_face(img, detector_backend = 'opencv'):
 	home = str(Path.home())
 	
 	if (detector_backend == 'opencv') or (detector_backend == 'ssd'):
-	
-		opencv_path = get_opencv_path()
-		eye_detector_path = opencv_path+"haarcascade_eye.xml"
-		eye_detector = cv2.CascadeClassifier(eye_detector_path)
 		
 		detected_face_gray = cv2.cvtColor(img, cv2.COLOR_BGR2GRAY) #eye detector expects gray scale image
 		
@@ -396,31 +388,10 @@ def align_face(img, detector_backend = 'opencv'):
 		return img #return img anyway
 	
 	elif detector_backend == 'dlib':
-	
-		#check required file exists in the home/.deepface/weights folder
-		
-		if os.path.isfile(home+'/.deepface/weights/shape_predictor_5_face_landmarks.dat') != True:
-			
-			print("shape_predictor_5_face_landmarks.dat.bz2 is going to be downloaded") 
-			
-			url = "http://dlib.net/files/shape_predictor_5_face_landmarks.dat.bz2"
-			output = home+'/.deepface/weights/'+url.split("/")[-1]
-			
-			gdown.download(url, output, quiet=False)
-			
-			zipfile = bz2.BZ2File(output)
-			data = zipfile.read()
-			newfilepath = output[:-4] #discard .bz2 extension
-			open(newfilepath, 'wb').write(data)
-		
-		#------------------------------
 		
 		import dlib #this is not a must dependency in deepface
 		
-		detector = dlib.get_frontal_face_detector()
-		sp = dlib.shape_predictor(home+"/.deepface/weights/shape_predictor_5_face_landmarks.dat")
-		
-		detections = detector(img, 1)
+		detections = face_detector(img, 1)
 		
 		if len(detections) > 0:
 			detected_face = detections[0]
@@ -431,9 +402,8 @@ def align_face(img, detector_backend = 'opencv'):
 	
 	elif detector_backend == 'mtcnn':
 		
-		from mtcnn import MTCNN #0.1.0
-		mtcnn_detector = MTCNN()
-		detections = mtcnn_detector.detect_faces(img)
+		img_rgb = cv2.cvtColor(img, cv2.COLOR_BGR2RGB) #mtcnn expects RGB but OpenCV read BGR
+		detections = face_detector.detect_faces(img_rgb)
 		
 		if len(detections) > 0:
 			detection = detections[0]
@@ -477,89 +447,3 @@ def preprocess_face(img, target_size=(224, 224), grayscale = False, enforce_dete
 	img_pixels /= 255 #normalize input in [0, 1]
 	
 	return img_pixels
-	
-def allocateMemory():
-	
-	#find allocated memories
-	gpu_indexes = []
-	memory_usage_percentages = []; available_memories = []; total_memories = []; utilizations = []
-	power_usages = []; power_capacities = []
-	
-	try:
-		result = subprocess.check_output(['nvidia-smi'])
-
-		dashboard = result.decode("utf-8").split("=|")
-
-		dashboard = dashboard[1].split("\n")
-		
-		gpu_idx = 0
-		for line in dashboard:
-			if ("MiB" in line):
-				power_info = line.split("|")[1]
-				power_capacity = int(power_info.split("/")[-1].replace("W", ""))
-				power_usage = int((power_info.split("/")[-2]).strip().split(" ")[-1].replace("W", ""))
-				
-				power_usages.append(power_usage)
-				power_capacities.append(power_capacity)
-				
-				#----------------------------
-				
-				memory_info = line.split("|")[2].replace("MiB","").split("/")
-				utilization_info = int(line.split("|")[3].split("%")[0])
-				
-				allocated = int(memory_info[0])
-				total_memory = int(memory_info[1])
-				available_memory = total_memory - allocated
-				
-				total_memories.append(total_memory)
-				available_memories.append(available_memory)
-				memory_usage_percentages.append(round(100*int(allocated)/int(total_memory), 4))
-				utilizations.append(utilization_info)
-				gpu_indexes.append(gpu_idx)
-				
-				gpu_idx = gpu_idx + 1
-		
-		gpu_count = gpu_idx * 1
-				
-	except Exception as err:
-		gpu_count = 0
-		#print(str(err))
-		
-	#------------------------------
-	
-	df = pd.DataFrame(gpu_indexes, columns = ["gpu_index"])
-	df["total_memories_in_mb"] = total_memories
-	df["available_memories_in_mb"] = available_memories
-	df["memory_usage_percentage"] = memory_usage_percentages
-	df["utilizations"] = utilizations
-	df["power_usages_in_watts"] = power_usages
-	df["power_capacities_in_watts"] = power_capacities
-	
-	df = df.sort_values(by = ["available_memories_in_mb"], ascending = False).reset_index(drop = True)
-	
-	#------------------------------
-	
-	required_memory = 10000 #All deepface models require 9016 MiB
-	
-	if df.shape[0] > 0: #has gpu
-		if df.iloc[0].available_memories_in_mb > required_memory:
-			my_gpu = str(int(df.iloc[0].gpu_index))
-			os.environ["CUDA_VISIBLE_DEVICES"] = my_gpu
-			
-			#------------------------------
-			#tf allocates all memory by default
-			#this block avoids greedy approach
-			
-			config = tf.ConfigProto()
-			config.gpu_options.allow_growth = True
-			session = tf.Session(config=config)
-			keras.backend.set_session(session)
-			
-			print("DeepFace will run on GPU (gpu_", my_gpu,")")
-		else:
-			#this case has gpu but no enough memory to allocate
-			os.environ["CUDA_VISIBLE_DEVICES"] = "" #run it on cpu
-			print("Even though the system has GPUs, there is no enough space in memory to allocate.")
-			print("DeepFace will run on CPU")
-	else:
-		print("DeepFace will run on CPU")
